// Copyright Hugh Perkins 2015 hughperkins at gmail
//
// This Source Code Form is subject to the terms of the Mozilla Public License, 
// v. 2.0. If a copy of the MPL was not distributed with this file, You can 
// obtain one at http://mozilla.org/MPL/2.0/.

// expected defines:
// one of: [ TANH | RELU | LINEAR | SIGMOID | SCALEDTANH ]

#ifdef TANH
    #define ACTIVATION_FUNCTION(output) (tanh(output))
#elif defined SCALEDTANH
    #define ACTIVATION_FUNCTION(output) ( 1.7159f * tanh( 0.66667f * output))
#elif SIGMOID
    #define ACTIVATION_FUNCTION(output) (1.0f / (1 + exp(-output)))
#elif defined RELU
    #define ACTIVATION_FUNCTION(output) (output> 0 ? output : 0)
#elif defined LINEAR
    #define ACTIVATION_FUNCTION(output) (output)
#endif

#ifdef ACTIVATION_FUNCTION // protect against not defined
kernel void activate( const int N, global float *inout ) {
    const int globalId = get_global_id(0);
    if( globalId >= N ) {
        return;
    }
    inout[globalId] = ACTIVATION_FUNCTION( inout[globalId] );
}
#endif

#ifdef ACTIVATION_FUNCTION // protect against not defined
kernel void forwardNaive( const int N, global float *out, global const float *in ) {
    const int globalId = get_global_id(0);
    if( globalId >= N ) {
        return;
    }
    out[globalId] = ACTIVATION_FUNCTION( in[globalId] );
}
#endif

